﻿using System;
using System.Drawing;
using System.Linq;
using System.Windows.Forms;

using Steema.TeeChart;

namespace tcharttest
{
    public partial class Form1 : Form
    {
        #region / Fields                    /

        System.Windows.Forms.Timer timer;
        int currentVertAxisIdx = -1;

        #endregion / Fields                    /
        #region / C'tor                     /

        /// <summary>
        /// Initializes a new instance of the <see cref="Form1"/> class.
        /// </summary>
        public Form1()
        {
            InitializeComponent();

            InitializeChart();

            timer = new System.Windows.Forms.Timer();
            timer.Interval = 2000; // every 2 seconds
            timer.Tick += new EventHandler(timer_Tick); // switches the current visible custom axis
            timer.Enabled = true;

            ForceDraw();
        }

        #endregion / C'tor                     /
        #region / Init                      /

        double[] penetrationValues = { -1.0,       0.0,       1.0,
                                        1.0e+014, -1.0e+014,
                                        /* this values lead to chart crashes
                                        1.0e+015, -1.0e+015, 
                                        1.0e+016, -1.0e+016,
                                        */
                                     };        

        private void InitializeChart()
        {
            tChart1.Aspect.View3D = false;

            int samples = 10;

            foreach (var val in penetrationValues)
            {
                var constantLine = new Steema.TeeChart.Styles.Line();
                var constantLineAxis = new Steema.TeeChart.Axis();

                constantLine.Chart = tChart1.Chart;
                SetAxisColor(constantLineAxis, constantLine.Color);

                constantLine.CustomVertAxis = constantLineAxis;

                constantLineAxis.Labels.Style = AxisLabelStyle.Value;

                tChart1.Series.Add(constantLine);
                tChart1.Axes.Custom.Add(constantLineAxis);

                for (int i = 0; i < samples; i++)
                    constantLine.Add(i, (double)val); // adds a constant signal
            }

            AddRefLine(samples);
        }

        private void AddRefLine(int samples)
        { 
            var referenceLine = new Steema.TeeChart.Styles.Line();
            var referenceAxis = new Steema.TeeChart.Axis();

            referenceLine.Chart = tChart1.Chart;
            SetAxisColor(referenceAxis, referenceLine.Color);

            referenceLine.CustomVertAxis = referenceAxis;

            referenceAxis.Labels.Style = AxisLabelStyle.Value;

            tChart1.Series.Add(referenceLine);
            tChart1.Axes.Custom.Add(referenceAxis);

            referenceLine.Add(0, -1.0e+014);
            referenceLine.Add(samples - 1, 1.0e+014);
        }

        private void SetAxisColor( Axis axis, Color color )
        {
            axis.AxisPen.Color      = color;
            axis.Ticks.Color        = color;
            axis.Labels.Font.Color  = color;
            axis.Title.Color        = color;
            axis.Title.Font.Color   = color;
        }

        #endregion / Init                      /
        #region / Properties                /

        Axis currentVertAxis = null;
        Axis ActiveVertAxis
        { 
            get
            {
                if (null != currentVertAxis)
                    return currentVertAxis;

                return this.tChart1.Axes.Left;
            }
            set{ currentVertAxis = value; }
        }

        #endregion / Properties                 /
        #region / Timer Handling            /

        void timer_Tick(object sender, EventArgs e)
        { // shuffles through the available axis
            foreach (var ax in tChart1.Axes.Custom.Cast<Axis>())
                ax.Visible = false;

            if (currentVertAxisIdx + 1 < tChart1.Axes.Custom.Count)
                currentVertAxisIdx++;
            else
                currentVertAxisIdx = 0;

            Axis currentAxis = tChart1.Axes.Custom[currentVertAxisIdx];
            currentAxis.Visible = true;
            ActiveVertAxis = currentAxis;

            ForceDraw();
        }

        #endregion / Timer Handling        /
        #region / Layout Adjustments        /

        internal void ForceDraw()
        {
            //TeeChart got a bug regarding internalDraw when window is not visible (e.g. the window is to small).
            // in this case a redraw is not needed as nothing is been seen anyhow and as soon as the window gets resised
            //the draw accours anyways
            try
            {
                if (chartHorizontalScrollPanel.Visible &&
                    (tChart1.Height > 1 && tChart1.Width > 1))
                {
                    tChart1.Invalidate(true);
                    CalcLeftAxisWidth();
                    tChart1.Draw();
                }
            }
            catch (Exception)
            {
                return;
            }
        }

        const int tchartsPanelMarginLeftOffset = 12; // in pixel
        private void CalcLeftAxisWidth()
        {
            tChart1.Panel.MarginLeft = ActiveVertAxis.MaxLabelsWidth() + tchartsPanelMarginLeftOffset;
        }

        #endregion / Layout Adjustments        /
        #region / Button Handling           /

        private void tChart1ShwoEditorButton_Click( object sender, EventArgs e )
        {
            tChart1.ShowEditor();
        }

        #endregion / Button Handling        /
    }
}